//@author: antokhio
//@help: 2d quad line with adjacency
//@tags: line
//@credits: Paul Houx
//https://github.com/paulhoux/Cinder-Samples/tree/master/GeometryShader

cbuffer cbPerDraw : register( b0 )
{
	float4x4 tWVP: WORLDVIEWPROJECTION;
	float4x4 tVI: VIEWINVERSE;
	float4x4 tW: WORLD;
};

cbuffer cbPerObj : register( b1 )
{
	float Thickness = 0.1f;
	float MITER_LIMIT = 0.75f;
	int IntanceStartIndex;
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
	float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 
};

StructuredBuffer<float3> lineBuffer;

struct VS_IN
{
	//float4 pos : POSITION;
	uint iv : SV_VertexID;
	uint ii : SV_InstanceID;
};

struct VS_OUT
{
    float4 pos : POSITION;
};

struct GS_OUT
{
    float4 pos : SV_POSITION;
};


VS_OUT VS(VS_IN input)
{
	VS_OUT output = (VS_OUT)0;
	output.pos = float4( lineBuffer[input.iv + IntanceStartIndex],1);
    //output.pos =  input.pos;//mul(input.pos,tWVP);
	
	return output;
}


float width = 0.1;

float2 find_perp (float2 p, float size)
{
	float v_lenght = sqrt(pow (p.x,2) + pow(p.y,2));
	float scale_factor = size / v_lenght;
	return float2 (-size * p.y, size * p.x);
}


void GS1(inout GS_OUT o, float4 p0, float4 p1, inout TriangleStream<GS_OUT> gsout)
{
	
	
	float3 cam = tVI[3].xyz;
	
	float3 dir = p1.xyz - p0.xyz;
	dir = normalize  (cross (dir, float3 (0,0,1)));
		
	o.pos = p0;
	o.pos.xy +=  width * dir.xy;
	gsout.Append(o);
	
	o.pos = p0;
	o.pos.xy += -width * dir.xy;
	gsout.Append(o);
	
	o.pos = p1;
	o.pos.xy += width * dir.xy;
	gsout.Append(o);
	
	o.pos = p1;
	o.pos.xy += -width * dir.xy;
	gsout.Append(o);
	
	gsout.RestartStrip();
	
}

[maxvertexcount(8)]
void GS_Diag(triangle VS_OUT input[3], inout TriangleStream<GS_OUT> gsout)
{
	GS_OUT o;
	float EPSILON = 0.01f;
	
	float3 t1 = input[0].pos.xyz;
    float3 t2 = input[1].pos.xyz;
	float3 t3 = input[2].pos.xyz;

	float3 l1 = t2 - t1;
	float3 l2 = t3 - t2;
	float3 l3 = t3 - t1;
	
	float dl1 = dot(l1,l1);
	float dl2 = dot(l2,l2);
	float dl3 = dot(l3,l3);
	
	float maxdistsqr = max(max(dl1,dl2),dl3);
	
	if (dl1 < maxdistsqr)
	{
		GS1( o, mul(float4(t1,1),tWVP),mul(float4(t2,1),tWVP), gsout);		
		//gsout.RestartStrip();
	}
		
	if (dl2 < maxdistsqr)
	{
		
		GS1( o, mul(float4(t3,1),tWVP),mul(float4(t2,1),tWVP), gsout);	
		//gsout.RestartStrip();
	}
	
	
	if (dl3 < maxdistsqr)
	{
		GS1( o, mul(float4(t3,1),tWVP),mul(float4(t1,1),tWVP), gsout);
		//gsout.RestartStrip();
	}
	
}



float4 PS(GS_OUT input) : SV_Target
{   
	float4 col = cAmb;
	col.a *= Alpha;
	return col;
}


technique11 Constant
{
	pass P0
	{

		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetGeometryShader(CompileShader(gs_5_0,GS_Diag() ) );
		//SetGeometryShader(CompileShader(gs_5_0,GS1() ) );
		SetPixelShader( CompileShader( ps_5_0, PS() ) );
	}
}



